@@ -7,7 +7,7 @@
 #include <QtCore/QWaitCondition>
 #include <QtCore/QAnimationDriver>
 #include <QtCore/QQueue>
-#include <QtCore/QTime>
+#include <QtCore/QTimer>
 
 #include <QtGui/QGuiApplication>
 #include <QtGui/QScreen>
@@ -18,6 +18,7 @@
 #include <QtQuick/QQuickWindow>
 #include <private/qquickwindow_p.h>
 #include <private/qquickitem_p.h>
+#include <QtGui/qpa/qplatformwindow_p.h>
 
 #include <QtQuick/private/qsgrenderer_p.h>
 
@@ -280,6 +281,7 @@ public:
     };
 
     void ensureRhi();
+    void teardownGraphics();
     void handleDeviceLoss();
 
     QSGThreadedRenderLoop *wm;
@@ -309,6 +311,7 @@ public:
     bool rhiDeviceLost = false;
     bool rhiDoomed = false;
     bool guiNotifiedAboutRhiFailure = false;
+    bool swRastFallbackDueToSwapchainFailure = false;
 
     // Local event queue stuff...
     bool stopEventProcessing;
@@ -575,20 +578,25 @@ void QSGRenderThread::sync(bool inExpose)
     }
 }
 
+void QSGRenderThread::teardownGraphics()
+{
+    QQuickWindowPrivate *wd = QQuickWindowPrivate::get(window);
+    wd->cleanupNodesOnShutdown();
+    sgrc->invalidate();
+    wm->releaseSwapchain(window);
+    if (ownRhi)
+        QSGRhiSupport::instance()->destroyRhi(rhi, {});
+    rhi = nullptr;
+}
+
 void QSGRenderThread::handleDeviceLoss()
 {
     if (!rhi || !rhi->isDeviceLost())
         return;
 
     qWarning("Graphics device lost, cleaning up scenegraph and releasing RHI");
-    QQuickWindowPrivate *wd = QQuickWindowPrivate::get(window);
-    wd->cleanupNodesOnShutdown();
-    sgrc->invalidate();
-    wm->releaseSwapchain(window);
+    teardownGraphics();
     rhiDeviceLost = true;
-    if (ownRhi)
-        QSGRhiSupport::instance()->destroyRhi(rhi, {});
-    rhi = nullptr;
 }
 
 void QSGRenderThread::syncAndRender()
@@ -620,6 +628,7 @@ void QSGRenderThread::syncAndRender()
     pendingUpdate = 0;
 
     QQuickWindowPrivate *cd = QQuickWindowPrivate::get(window);
+    QSGRhiSupport *rhiSupport = QSGRhiSupport::instance();
     // Begin the frame before syncing -> sync is where we may invoke
     // updatePaintNode() on the items and they may want to do resource updates.
     // Also relevant for applications that connect to the before/afterSynchronizing
@@ -641,10 +650,29 @@ void QSGRenderThread::syncAndRender()
                 qCDebug(QSG_LOG_RENDERLOOP, QSG_RT_PAD, "just became exposed");
 
             cd->hasActiveSwapchain = cd->swapchain->createOrResize();
-            if (!cd->hasActiveSwapchain && rhi->isDeviceLost()) {
-                handleDeviceLoss();
-                QCoreApplication::postEvent(window, new QEvent(QEvent::Type(QQuickWindowPrivate::FullUpdateRequest)));
-                return;
+            if (!cd->hasActiveSwapchain) {
+                bool bailOut = false;
+                if (rhi->isDeviceLost()) {
+                    handleDeviceLoss();
+                    bailOut = true;
+                } else if (previousOutputSize.isEmpty() && !swRastFallbackDueToSwapchainFailure && rhiSupport->attemptReinitWithSwRastUponFail()) {
+                    qWarning("Failed to create swapchain."
+                             " Retrying by requesting a software rasterizer, if applicable for the 3D API implementation.");
+                    swRastFallbackDueToSwapchainFailure = true;
+                    teardownGraphics();
+                    bailOut = true;
+                }
+                if (bailOut) {
+                    QCoreApplication::postEvent(window, new QEvent(QEvent::Type(QQuickWindowPrivate::FullUpdateRequest)));
+                    if (syncRequested) {
+                        // Lock like sync() would do. Note that exposeRequested always includes syncRequested.
+                        qCDebug(QSG_LOG_RENDERLOOP, QSG_RT_PAD, "- bailing out due to failed swapchain init, wake Gui");
+                        mutex.lock();
+                        waitCondition.wakeOne();
+                        mutex.unlock();
+                    }
+                    return;
+                }
             }
 
             cd->swapchainJustBecameRenderable = false;
@@ -851,7 +879,8 @@ void QSGRenderThread::ensureRhi()
         if (rhiDoomed) // no repeated attempts if the initial attempt failed
             return;
         QSGRhiSupport *rhiSupport = QSGRhiSupport::instance();
-        QSGRhiSupport::RhiCreateResult rhiResult = rhiSupport->createRhi(window, offscreenSurface);
+        const bool forcePreferSwRenderer = swRastFallbackDueToSwapchainFailure;
+        QSGRhiSupport::RhiCreateResult rhiResult = rhiSupport->createRhi(window, offscreenSurface, forcePreferSwRenderer);
         rhi = rhiResult.rhi;
         ownRhi = rhiResult.own;
         if (rhi) {
@@ -1433,10 +1462,29 @@ void QSGThreadedRenderLoop::update(QQuickWindow *window)
     if (!w)
         return;
 
-    if (w->thread == QThread::currentThread()) {
-        qCDebug(QSG_LOG_RENDERLOOP) << "update on window - on render thread" << w->window;
-        w->thread->requestRepaint();
-        return;
+    const bool isRenderThread = QThread::currentThread() == w->thread;
+
+#if defined(Q_OS_MACOS)
+    using namespace QNativeInterface::Private;
+    if (auto *cocoaWindow = dynamic_cast<QCocoaWindow*>(window->handle())) {
+        // If the window is being resized we don't want to schedule unthrottled
+        // updates on the render thread, as this will starve the main thread
+        // from getting drawables for displaying the updated window size.
+        if (isRenderThread && cocoaWindow->inLiveResize()) {
+            // In most cases the window will already have update requested
+            // due to the animator triggering a sync, but just in case we
+            // schedule an update request on the main thread explicitly.
+            qCDebug(QSG_LOG_RENDERLOOP) << "window is resizing. update on window" << w->window;
+            QTimer::singleShot(0, window, [=]{ window->requestUpdate(); });
+            return;
+        }
+    }
+#endif
+
+    if (isRenderThread) {
+       qCDebug(QSG_LOG_RENDERLOOP) << "update on window - on render thread" << w->window;
+       w->thread->requestRepaint();
+       return;
     }
 
     qCDebug(QSG_LOG_RENDERLOOP) << "update on window" << w->window;
@@ -1636,25 +1684,47 @@ void QSGThreadedRenderLoop::polishAndSync(Window *w, bool inExpose)
     // isVSyncDependent() == true, if not then we always use the driver and
     // just advance here)
     if (m_animation_timer == 0 && m_animation_driver->isRunning()) {
-        qCDebug(QSG_LOG_RENDERLOOP, "- advancing animations");
-        m_animation_driver->advance();
-        qCDebug(QSG_LOG_RENDERLOOP, "- animations done..");
+        auto advanceAnimations = [this, window=QPointer(window)] {
+            qCDebug(QSG_LOG_RENDERLOOP, "- advancing animations");
+            m_animation_driver->advance();
+            qCDebug(QSG_LOG_RENDERLOOP, "- animations done..");
 
-        // We need to trigger another update round to keep all animations
-        // running correctly. For animations that lead to a visual change (a
-        // property change in some item leading to dirtying the item and so
-        // ending up in maybeUpdate()) this would not be needed, but other
-        // animations would then stop functioning since there is nothing
-        // advancing the animation system if we do not call postUpdateRequest()
-        // here and nothing else leads to it either. This has an unfortunate
-        // side effect in multi window cases: one can end up in a situation
-        // where a non-animating window gets updates continuously because there
-        // is an animation running in some other window that is non-exposed or
-        // even closed already (if it was exposed we would not hit this branch,
-        // however). Sadly, there is nothing that can be done about it.
-        postUpdateRequest(w);
+            // We need to trigger another update round to keep all animations
+            // running correctly. For animations that lead to a visual change (a
+            // property change in some item leading to dirtying the item and so
+            // ending up in maybeUpdate()) this would not be needed, but other
+            // animations would then stop functioning since there is nothing
+            // advancing the animation system if we do not call postUpdateRequest()
+            // here and nothing else leads to it either. This has an unfortunate
+            // side effect in multi window cases: one can end up in a situation
+            // where a non-animating window gets updates continuously because there
+            // is an animation running in some other window that is non-exposed or
+            // even closed already (if it was exposed we would not hit this branch,
+            // however). Sadly, there is nothing that can be done about it.
+            if (window)
+                window->requestUpdate();
 
-        emit timeToIncubate();
+            emit timeToIncubate();
+        };
+
+#if defined(Q_OS_APPLE)
+        if (inExpose) {
+            // If we are handling an expose event the system is expecting us to
+            // produce a frame that it can present on screen to the user. Advancing
+            // animations at this point might result in changing properties of the
+            // window in a way that invalidates the current frame, resulting in the
+            // discarding of the current frame before the user ever sees it. To give
+            // the system a chance to present the current frame we defer the advance
+            // of the animations until the start of the next event loop pass, which
+            // should still give plenty of time to compute the new render state before
+            // the next expose event or update request.
+            QMetaObject::invokeMethod(this, advanceAnimations, Qt::QueuedConnection);
+        } else
+#endif // Q_OS_APPLE
+        {
+            // For regular update requests we assume we can advance here synchronously
+            advanceAnimations();
+        }
     } else if (w->updateDuringSync) {
         postUpdateRequest(w);
     }
@@ -1686,6 +1756,7 @@ bool QSGThreadedRenderLoop::event(QEvent *e)
             emit timeToIncubate();
             return true;
         }
+        break;
     }
 
     default:
