@@ -3,10 +3,10 @@
 
 /*---
 esid: sec-array.prototype.some
-es5id: 15.4.4.17-3-15
+es5id: 15.4.4.17-2-7
 description: >
-    Array.prototype.some - 'length' is a string containing an
-    exponential number
+    Array.prototype.some - 'length' is an own accessor property on an
+    Array-like object
 ---*/
 
 function callbackfn1(val, idx, obj) {
@@ -17,12 +17,18 @@ function callbackfn2(val, idx, obj) {
   return val > 11;
 }
 
-var obj = {
-  0: 9,
-  1: 11,
-  2: 12,
-  length: "2E0"
-};
+var obj = {};
+
+Object.defineProperty(obj, "length", {
+  get: function() {
+    return 2;
+  },
+  configurable: true
+});
+
+obj[0] = 9;
+obj[1] = 11;
+obj[2] = 12;
 
 assert(Array.prototype.some.call(obj, callbackfn1), 'Array.prototype.some.call(obj, callbackfn1) !== true');
 assert.sameValue(Array.prototype.some.call(obj, callbackfn2), false, 'Array.prototype.some.call(obj, callbackfn2)');
