@@ -36,61 +36,44 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '15.9.1.1-1.js';
+gTestfile = '15.9.5.13-1.js';
 
 /**
-   File Name:          15.9.1.1-1.js
-   ECMA Section:       15.9.1.1 Time Range
-   Description:
-   - leap seconds are ignored
-   - assume 86400000 ms / day
-   - numbers range fom +/- 9,007,199,254,740,991
-   - ms precision for any instant that is within
-   approximately +/-285,616 years from 1 jan 1970
-   UTC
-   - range of times supported is -100,000,000 days
-   to 100,000,000 days from 1 jan 1970 12:00 am
-   - time supported is 8.64e5*10e8 milliseconds from
-   1 jan 1970 UTC  (+/-273972.6027397 years)
+   File Name:          15.9.5.13.js
+   ECMA Section:       15.9.5.13
+   Description:        Date.prototype.getUTCDay
+
+   1.Let t be this time value.
+   2.If t is NaN, return NaN.
+   3.Return WeekDay(t).
 
-   -   this test generates its own data -- it does not
-   read data from a file.
    Author:             christine@netscape.com
-   Date:               7 july 1997
-
-   Static variables:
-   FOUR_HUNDRED_YEARS
-
+   Date:               12 november 1997
 */
 
-//  every one hundred years contains:
-//    24 years with 366 days
-//
-//  every four hundred years contains:
-//    97 years with 366 days
-//   303 years with 365 days
-//
-//   86400000*365*97    =    3067372800000
-//  +86400000*366*303   =  + 9555408000000
-//                      =    1.26227808e+13
-var FOUR_HUNDRED_YEARS = 1.26227808e+13;
-var SECTION         =  "15.9.1.1-1";
+var SECTION = "15.9.5.13";
+var VERSION = "ECMA_1";
 startTest();
+var TITLE   = "Date.prototype.getUTCDay()";
 
-writeHeaderToLog("15.8.1.1 Time Range");
+writeHeaderToLog( SECTION + " "+ TITLE);
 
-var M_SECS;
-var CURRENT_YEAR;
+// get the current time
+var now = (new Date()).valueOf();
 
-for ( M_SECS = 0, CURRENT_YEAR = 1970;
-      M_SECS < 8640000000000000;
-      M_SECS += FOUR_HUNDRED_YEARS, CURRENT_YEAR += 400 ) {
-
-  new TestCase( SECTION,  
-		"new Date("+M_SECS+")",
-		CURRENT_YEAR,
-		(new Date( M_SECS)).getUTCFullYear() );
-}
+addTestCase( now );
 
 test();
 
+function addTestCase( t ) {
+  var start = TimeFromYear(YearFromTime(t));
+  var stop  = TimeFromYear(YearFromTime(t) + 1);
+
+  for (var d = start; d < stop; d += msPerDay)
+  {
+    new TestCase( SECTION,
+                  "(new Date("+d+")).getUTCDay()",
+                  WeekDay((d)),
+                  (new Date(d)).getUTCDay() );
+  }
+}
